@@ -1,7 +1,9 @@
+use chrono::{DateTime, Utc};
+use ic_cdk::api::time;
 use ic_cdk::export::candid::{CandidType, Deserialize, Principal};
 use ic_cdk::{caller, print};
 use ic_event_hub_macros::Event;
-use std::time::{SystemTime, UNIX_EPOCH};
+use std::time::{Duration, SystemTime, UNIX_EPOCH};
 
 #[derive(Event, CandidType, Deserialize)]
 pub struct VotingPowerUpdateEvent {
@@ -28,10 +30,24 @@ pub struct RemoteCallPayload {
     pub cycles: u64,
 }
 
+#[inline(always)]
 pub fn log(msg: &str) {
-    print(format!("[caller: {}]: {}", caller(), msg))
+    print(format!(
+        "{}: [caller: {}]: {}",
+        make_time(time()),
+        caller(),
+        msg
+    ))
 }
 
+fn make_time(nanos: u64) -> String {
+    let d = UNIX_EPOCH + Duration::from_nanos(nanos);
+    let datetime = DateTime::<Utc>::from(d);
+
+    datetime.format("%Y-%m-%d %H:%M:%S.%f").to_string()
+}
+
+#[inline(always)]
 pub fn random_principal_test() -> Principal {
     Principal::from_slice(
         &SystemTime::now()
@@ -40,4 +56,4 @@ pub fn random_principal_test() -> Principal {
             .as_nanos()
             .to_be_bytes(),
     )
-}
\ No newline at end of file
+}
